/*
 * Copyright (c) 2006 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: JythonOneTimeRunnerImpl.java,v 1.2 2006/11/20 18:35:10 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.jython.runner;

import java.io.File;
import java.security.AccessController;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import java.util.jar.JarFile;

import org.python.core.PySystemState;
import org.python.util.PythonInterpreter;

import xsul5.MLogger;

import edu.indiana.extreme.xbaya.XBayaException;

/**
 * @author Satoshi Shirasuna
 */
public class JythonOneTimeRunnerImpl implements JythonOneTimeRunner {

    private static final MLogger logger = MLogger.getLogger();

    /**
     * @throws XBayaException
     * @see edu.indiana.extreme.xbaya.jython.runner.JythonOneTimeRunner#run(java.lang.String,
     *      java.lang.String[])
     */
    public void run(final String script, final String[] arguments)
            throws XBayaException {

        try {
            AccessController
                    .doPrivileged(new PrivilegedExceptionAction<Void>() {
                        public Void run() {
                            ClassLoader loader = this.getClass()
                                    .getClassLoader();

                            PySystemState.initialize(System.getProperties(),
                                    null, arguments, loader);

                            if (loader instanceof JythonClassLoader) {
                                logger.info("jythonClassLoader");
                                JythonClassLoader jythonLoader = (JythonClassLoader) loader;

                                JarFile xbayaJarFile = jythonLoader
                                        .getXBayaJarFile();
                                if (xbayaJarFile != null) {
                                    String jarPath = xbayaJarFile.getName();
                                    logger.finest("jarPath: " + jarPath);
                                    // String jarDir = jarPath.substring(0,
                                    // jarPath.lastIndexOf());
                                    File jarFile = new File(jarPath);
                                    String jarDir = jarFile.getParent();
                                    logger.finest("jarDir: " + jarDir);

                                    // This is for the Jython interpreter to
                                    // solve import statements.
                                    PySystemState.add_extdir(jarDir);
                                }
                            }

                            PythonInterpreter interpreter = new PythonInterpreter();
                            interpreter.exec(script);
                            return null;
                        }
                    });
        } catch (PrivilegedActionException e) {
            logger.caught(e);
            throw new XBayaException(e.getCause());
        }

    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2006 The Trustees of Indiana University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
